@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.28.2.2 2003/04/11 14:02:35 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.28.2.5 2003/08/09 13:27:08 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -43,6 +43,8 @@
 #include <net/route.h>                  /* for ip_route_output */
 #include <linux/netfilter.h>
 #include <linux/netfilter_ipv4.h>
+#include <linux/jhash.h>
+#include <linux/random.h>
 
 #include "ip_vs.h"
 //#include <net/ip_vs.h>
@@ -53,17 +55,17 @@
  */
 static struct list_head *ip_vs_conn_tab;
 
-/*  SLAB cache for IPVS connections */
+/* SLAB cache for IPVS connections */
 static kmem_cache_t *ip_vs_conn_cachep;
 
-/*  counter for current IPVS connections */
+/* counter for current IPVS connections */
 static atomic_t ip_vs_conn_count = ATOMIC_INIT(0);
 
-/*
- *  No client port connection counter
- */
+/* counter for no-client-port connections */
 static atomic_t ip_vs_conn_no_cport_cnt = ATOMIC_INIT(0);
 
+/* random value for IPVS connection hash */
+static unsigned int ip_vs_conn_rnd;
 
 /*
  *  Fine locking granularity for big connection hash table
@@ -125,12 +127,10 @@ static inline void ct_write_unlock_bh(unsigned key)
 /*
  *	Returns hash value for IPVS connection entry
  */
-static inline unsigned
+static unsigned
 ip_vs_conn_hashkey(unsigned proto, __u32 addr, __u16 port)
 {
-	unsigned addrh = ntohl(addr);
-
-	return (proto^addrh^(addrh>>IP_VS_CONN_TAB_BITS)^ntohs(port))
+	return jhash_3words(addr, port, proto, ip_vs_conn_rnd)
 		& IP_VS_CONN_TAB_MASK;
 }
 
@@ -297,7 +297,7 @@ struct ip_vs_conn *ip_vs_conn_out_get
 void ip_vs_conn_put(struct ip_vs_conn *cp)
 {
 	/* reset it expire in its timeout */
-	mod_sltimer(&cp->timer, jiffies+cp->timeout);
+	mod_timer(&cp->timer, jiffies+cp->timeout);
 
 	__ip_vs_conn_put(cp);
 }
@@ -628,10 +628,8 @@ static int ip_vs_bypass_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 		goto tx_error;
 	}
 
-#if 0
-	if (skb_is_nonlinear(skb) && skb->len <= mtu)
-#endif
-		ip_send_check(iph);
+	/* update checksum because skb might be defragmented */
+	ip_send_check(iph);
 
 	if (unlikely(skb_headroom(skb) < rt->u.dst.dev->hard_header_len)) {
 		if (skb_cow(skb, rt->u.dst.dev->hard_header_len)) {
@@ -890,10 +888,8 @@ static int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 		goto tx_error;
 	}
 
-#if 0
-	if (skb_is_nonlinear(skb))
-#endif
-		ip_send_check(old_iph);
+	/* update checksum because skb might be defragmented */
+	ip_send_check(old_iph);
 
 	skb->h.raw = skb->nh.raw;
 
@@ -914,6 +910,7 @@ static int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 		}
 		kfree_skb(skb);
 		skb = new_skb;
+		old_iph = skb->nh.iph;
 	}
 
 	skb->nh.raw = skb_push(skb, sizeof(struct iphdr));
@@ -981,10 +978,8 @@ static int ip_vs_dr_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 		goto tx_error;
 	}
 
-#if 0
-	if (skb_is_nonlinear(skb) && skb->len <= mtu)
-#endif
-		ip_send_check(iph);
+	/* update checksum because skb might be defragmented */
+	ip_send_check(iph);
 
 	if (unlikely(skb_headroom(skb) < rt->u.dst.dev->hard_header_len)) {
 		if (skb_cow(skb, rt->u.dst.dev->hard_header_len)) {
@@ -1060,9 +1055,7 @@ ip_vs_bind_dest(struct ip_vs_conn *cp, struct ip_vs_dest *dest)
 	if (!dest)
 		return;
 
-	/*
-	 *    Increase the refcnt counter of the dest.
-	 */
+	/* Increase the refcnt counter of the dest */
 	atomic_inc(&dest->refcnt);
 
 	/* Bind with the destination and its corresponding transmitter */
@@ -1089,38 +1082,40 @@ static inline void ip_vs_unbind_dest(struct ip_vs_conn *cp)
 {
 	struct ip_vs_dest *dest = cp->dest;
 
-	if (dest) {
-		IP_VS_DBG(9, "Unbind-dest %s c:%u.%u.%u.%u:%d "
-			  "v:%u.%u.%u.%u:%d d:%u.%u.%u.%u:%d fwd:%c "
-			  "s:%s flg:%X cnt:%d destcnt:%d",
-			  ip_vs_proto_name(cp->protocol),
-			  NIPQUAD(cp->caddr), ntohs(cp->cport),
-			  NIPQUAD(cp->vaddr), ntohs(cp->vport),
-			  NIPQUAD(cp->daddr), ntohs(cp->dport),
-			  ip_vs_fwd_tag(cp), ip_vs_state_name(cp->state),
-			  cp->flags, atomic_read(&cp->refcnt),
-			  atomic_read(&dest->refcnt));
+	/* if dest is NULL, then return directly */
+	if (!dest)
+		return;
 
-		/*
-		 * Decrease the inactconns or activeconns counter
-		 * if it is not a connection template ((cp->cport!=0)
-		 *   || (cp->flags & IP_VS_CONN_F_NO_CPORT)).
-		 */
-		if (cp->cport || (cp->flags & IP_VS_CONN_F_NO_CPORT)) {
-			if (cp->flags & IP_VS_CONN_F_INACTIVE) {
-				atomic_dec(&dest->inactconns);
-			} else {
-				atomic_dec(&dest->activeconns);
-			}
+	IP_VS_DBG(9, "Unbind-dest %s c:%u.%u.%u.%u:%d "
+		  "v:%u.%u.%u.%u:%d d:%u.%u.%u.%u:%d fwd:%c "
+		  "s:%s flg:%X cnt:%d destcnt:%d",
+		  ip_vs_proto_name(cp->protocol),
+		  NIPQUAD(cp->caddr), ntohs(cp->cport),
+		  NIPQUAD(cp->vaddr), ntohs(cp->vport),
+		  NIPQUAD(cp->daddr), ntohs(cp->dport),
+		  ip_vs_fwd_tag(cp), ip_vs_state_name(cp->state),
+		  cp->flags, atomic_read(&cp->refcnt),
+		  atomic_read(&dest->refcnt));
+
+	/*
+	 * Decrease the inactconns or activeconns counter
+	 * if it is not a connection template ((cp->cport!=0)
+	 *   || (cp->flags & IP_VS_CONN_F_NO_CPORT)).
+	 */
+	if (cp->cport || (cp->flags & IP_VS_CONN_F_NO_CPORT)) {
+		if (cp->flags & IP_VS_CONN_F_INACTIVE) {
+			atomic_dec(&dest->inactconns);
+		} else {
+			atomic_dec(&dest->activeconns);
 		}
-
-		/*
-		 * Simply decrease the refcnt of the dest, because the
-		 * dest will be either in service's destination list
-		 * or in the trash.
-		 */
-		atomic_dec(&dest->refcnt);
 	}
+
+	/*
+	 * Simply decrease the refcnt of the dest, because the
+	 * dest will be either in service's destination list
+	 * or in the trash.
+	 */
+	atomic_dec(&dest->refcnt);
 }
 
 
@@ -1215,7 +1210,7 @@ static void ip_vs_conn_expire(unsigned long data)
 	if (likely(atomic_read(&cp->refcnt) == 1)) {
 		/* make sure that there is no timer on it now */
 		if (timer_pending(&cp->timer))
-			del_sltimer(&cp->timer);
+			del_timer(&cp->timer);
 
 		/* does anybody control me? */
 		if (cp->control)
@@ -1247,7 +1242,7 @@ static void ip_vs_conn_expire(unsigned long data)
 void ip_vs_conn_expire_now(struct ip_vs_conn *cp)
 {
 	cp->timeout = 0;
-	mod_sltimer(&cp->timer, jiffies);
+	mod_timer(&cp->timer, jiffies);
 	__ip_vs_conn_put(cp);
 }
 
@@ -1525,15 +1520,6 @@ int ip_vs_conn_init(void)
 	if (!ip_vs_conn_tab)
 		return -ENOMEM;
 
-	/* Allocate ip_vs_conn slab cache */
-	ip_vs_conn_cachep = kmem_cache_create("ip_vs_conn",
-					      sizeof(struct ip_vs_conn), 0,
-					      SLAB_HWCACHE_ALIGN, NULL, NULL);
-	if (!ip_vs_conn_cachep) {
-		vfree(ip_vs_conn_tab);
-		return -ENOMEM;
-	}
-
 	IP_VS_INFO("Connection hash table configured "
 		   "(size=%d, memory=%ldKbytes)\n",
 		   IP_VS_CONN_TAB_SIZE,
@@ -1549,8 +1535,20 @@ int ip_vs_conn_init(void)
 		__ip_vs_conntbl_lock_array[idx].l = RW_LOCK_UNLOCKED;
 	}
 
+	/* Allocate ip_vs_conn slab cache */
+	ip_vs_conn_cachep = kmem_cache_create("ip_vs_conn",
+					      sizeof(struct ip_vs_conn), 0,
+					      SLAB_HWCACHE_ALIGN, NULL, NULL);
+	if (!ip_vs_conn_cachep) {
+		vfree(ip_vs_conn_tab);
+		return -ENOMEM;
+	}
+
 	proc_net_create("ip_vs_conn", 0, ip_vs_conn_getinfo);
 
+	/* calculate the random value for connection hash */
+	get_random_bytes(&ip_vs_conn_rnd, sizeof(ip_vs_conn_rnd));
+
 	return 0;
 }
 
