@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_core.c,v 1.31.2.2 2003/04/11 14:02:35 wensong Exp $
+ * Version:     $Id: ip_vs_core.c,v 1.31.2.5 2003/07/29 14:37:12 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -844,10 +844,11 @@ unsigned int check_for_ip_vs_out(struct sk_buff **skb_p,
 
 
 /*
- *	Handle ICMP messages in the outside-to-inside direction (incoming).
+ *	Handle ICMP messages in the outside-to-inside direction (incoming)
+ *	and sometimes in outgoing direction from ip_vs_forward_icmp.
  *	Find any that might be relevant, check against existing connections,
  *	forward to the right destination host if relevant.
- *	Currently handles error types - unreachable, quench, ttl exceeded
+ *	Currently handles error types - unreachable, quench, ttl exceeded.
  */
 static int ip_vs_in_icmp(struct sk_buff **skb_p)
 {
@@ -865,14 +866,11 @@ static int ip_vs_in_icmp(struct sk_buff **skb_p)
 	if (skb_is_nonlinear(skb)) {
 		if (skb_linearize(skb, GFP_ATOMIC) != 0)
 			return NF_DROP;
-#if 0
-		ip_send_check(skb->nh.iph);
-#endif
 	}
 
 	iph = skb->nh.iph;
 	ip_send_check(iph);
-	icmph = (struct icmphdr *)((char *)iph+(iph->ihl<<2));
+	icmph = (struct icmphdr *)((char *)iph + (iph->ihl << 2));
 	len = ntohs(iph->tot_len) - (iph->ihl<<2);
 	if (len < sizeof(struct icmphdr))
 		return NF_DROP;
@@ -1218,12 +1216,10 @@ static int __init ip_vs_init(void)
 		goto cleanup_nothing;
 	}
 
-	ip_vs_sltimer_init();
-
 	ret = ip_vs_conn_init();
 	if (ret < 0) {
 		IP_VS_ERR("can't setup connection table.\n");
-		goto cleanup_sltimer;
+		goto cleanup_control;
 	}
 
 	ret = ip_vs_app_init();
@@ -1266,8 +1262,7 @@ static int __init ip_vs_init(void)
 	ip_vs_app_cleanup();
   cleanup_conn:
 	ip_vs_conn_cleanup();
-  cleanup_sltimer:
-	ip_vs_sltimer_cleanup();
+  cleanup_control:
 	ip_vs_control_cleanup();
   cleanup_nothing:
 	return ret;
@@ -1281,7 +1276,6 @@ static void __exit ip_vs_cleanup(void)
 	nf_unregister_hook(&ip_vs_in_ops);
 	ip_vs_app_cleanup();
 	ip_vs_conn_cleanup();
-	ip_vs_sltimer_cleanup();
 	ip_vs_control_cleanup();
 	IP_VS_INFO("ipvs unloaded.\n");
 }
