@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.30.2.1 2002/11/14 10:05:23 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.30.2.3 2003/07/29 14:37:12 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -93,7 +93,7 @@ int ip_vs_get_debug_level(void)
 /*
  *  update_defense_level is called from timer bh and from sysctl.
  */
-void update_defense_level(void)
+static void update_defense_level(void)
 {
 	int ip_vs_amem = nr_free_pages() + atomic_read(&page_cache_size) +
 		atomic_read(&buffermem_pages);
@@ -189,6 +189,22 @@ void update_defense_level(void)
 }
 
 
+/*
+ *	Timer for checking the defense
+ */
+static struct timer_list defense_timer;
+#define DEFENSE_TIMER_PERIOD	1*HZ
+
+static void defense_timer_handler(unsigned long data)
+{
+	update_defense_level();
+	if (atomic_read(&ip_vs_dropentry))
+		ip_vs_random_dropentry();
+
+	mod_timer(&defense_timer, jiffies + DEFENSE_TIMER_PERIOD);
+}
+
+
 /*
  *  Hash table: for virtual service lookups
  */
@@ -846,7 +862,7 @@ static int ip_vs_edit_dest(struct ip_vs_service *svc,
 	EnterFunction(2);
 
 	if (ur->weight < 0) {
-		IP_VS_ERR("ip_vs_add_dest(): server weight less than zero\n");
+		IP_VS_ERR("ip_vs_edit_dest(): server weight less than zero\n");
 		return -ERANGE;
 	}
 
@@ -2109,6 +2125,12 @@ int ip_vs_control_init(void)
 	ip_vs_stats.lock = SPIN_LOCK_UNLOCKED;
 	ip_vs_new_estimator(&ip_vs_stats);
 
+	/* Hook the defense timer */
+	init_timer(&defense_timer);
+	defense_timer.function = defense_timer_handler;
+	defense_timer.expires = jiffies + DEFENSE_TIMER_PERIOD;
+	add_timer(&defense_timer);
+
 	LeaveFunction(2);
 	return 0;
 }
@@ -2117,6 +2139,7 @@ void ip_vs_control_cleanup(void)
 {
 	EnterFunction(2);
 	ip_vs_trash_cleanup();
+	del_timer_sync(&defense_timer);
 	ip_vs_kill_estimator(&ip_vs_stats);
 	unregister_sysctl_table(ipv4_vs_table.sysctl_header);
 	proc_net_remove("ip_vs_stats");
