@@ -1,7 +1,7 @@
 /*
  * ip_vs_est.c  Simple rate estimator for IPVS
  *
- * Version:     $Id: ip_vs_est.c,v 1.2 2001/11/04 08:58:43 wensong Exp $
+ * Version:     $Id: ip_vs_est.c,v 1.3.2.1 2003/07/29 14:37:13 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -21,7 +21,7 @@
 
 /*
   This code is to estimate rate in a shorter interval (such as 8
-  seconds)for virtual services and real servers. For measure rate in a
+  seconds) for virtual services and real servers. For measure rate in a
   long interval, it is easy to implement a user level daemon which
   periodically reads those statistical counters and measure rate.
 
@@ -78,6 +78,8 @@ static void estimation_timer(unsigned long arg)
 	read_lock(&est_lock);
 	for (e = est_list; e; e = e->next) {
 		s = e->stats;
+
+		spin_lock(&s->lock);
 		n_conns = s->conns;
 		n_inpkts = s->inpkts;
 		n_outpkts = s->outpkts;
@@ -109,6 +111,7 @@ static void estimation_timer(unsigned long arg)
 		e->last_outbytes = n_outbytes;
 		e->outbps += ((long)rate - (long)e->outbps)>>2;
 		s->outbps = (e->outbps+0xF)>>5;
+		spin_unlock(&s->lock);
 	}
 	read_unlock(&est_lock);
 	mod_timer(&est_timer, jiffies + 2*HZ);
